<?php
/*
    Plugin Name: WooCommerce Smartcash Payment Gateway
    Plugin URI: http://www.smart-cash.it
    Description: Smartcash - Payment gateway for woocommerce
    Version: 2.1.4
    Author: Stonewall Capital SRL
    Author URI: http://www.stonewallcapital.it
*/


define("SC_PROXY_TYPE",CURLPROXY_SOCKS5);


if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    add_action('plugins_loaded', 'woocommerce_smartpay_init', 0);
    add_action('init', 'check_for_payment');

    function check_for_payment()
    {
        WC()->payment_gateways();
        do_action('check_smartpay_payment');
    }

    function woocommerce_smartpay_init()
    {
        if (!class_exists('WC_Payment_Gateway')) return;

        class WC_SmartPay extends WC_Payment_Gateway
        {
            public function __construct()
            {
                $this->id            = 'smartpay';
                $this->medthod_title = 'smartpay';
                $this->has_fields    = false;

                $this->init_form_fields();
                $this->init_settings();

                $this->title                    = "Credit Card Payment";
                $this->smartpay_merchant_key    = $this->settings['smartpay_merchant_key'];
                $this->smartpay_merchant_secret = $this->settings['smartpay_merchant_secret'];
                $this->smartpay_proxy_url = $this->settings['smartpay_proxy_url'];
                $this->smartpay_proxy_login = $this->settings['smartpay_proxy_login'];
                $this->smartpay_proxy_password = $this->settings['smartpay_proxy_password'];
                $this->smartpay_proxy_active = $this->settings['smartpay_proxy_active'];
                $this->initializePaymentUrl     = 'https://www.smart-cash.it/smartPay/initializePayment.php';
                $this->getTransactionStatusUrl  = 'https://www.smart-cash.it/smartPay/getTransactionStatus.php';

                $this->msg['message'] = "";
                $this->msg['class']   = "";

                add_action('check_smartpay_payment', array(&$this, 'check_smartpay_response'));

                //add_action('woocommerce_payment_complete_order_status', array(
                //    &$this,
                //    'order_completed_callback'
                //), /*priority*/ 10, /*arguments*/ 2);

                if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
                    add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(
                        &$this,
                        'process_admin_options'
                    ));
                } else {
                    add_action('woocommerce_update_options_payment_gateways', array(
                        &$this,
                        'process_admin_options'
                    ));
                }

                add_action('woocommerce_receipt_smartpay', array(&$this, 'receipt_page'));
            }

            function init_form_fields()
            {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __('Attiva/Disattiva', 'smartpay'),
                        'type' => 'checkbox',
                        'label' => __('Abilita il modulo smartPay.', 'smartpay'),
                        'default' => 'no'
                    ),
                    'smartpay_merchant_key' => array(
                        'title' => __('Merchant Key:', 'smartpay'),
                        'type' => 'text',
                        'description' => __('', 'smartpay')
                    ),
                    'smartpay_merchant_secret' => array(
                        'title' => __('Merchant Secret:', 'smartpay'),
                        'type' => 'text',
                        'description' => __('')
                    ),
                    'smartpay_proxy_url' => array(
                        'title' => __('Proxy URL:', 'smartpay'),
                        'type' => 'text',
                        'description' => __('')
                    ),
                    'smartpay_proxy_login' => array(
                        'title' => __('Proxy login:', 'smartpay'),
                        'type' => 'text',
                        'description' => __('')
                    ),
                    'smartpay_proxy_password' => array(
                        'title' => __('Proxy Secret:', 'smartpay'),
                        'type' => 'text',
                        'description' => __('')
                    ),
                    'smartpay_proxy_active' => array(
                        'title' => __('Proxy attivo', 'smartpay'),
                        'type' => 'checkbox',
                        'description' => __('')
                    )
                );
            }

            public function admin_options()
            {
                echo '<h3>' . __('smartPay Payment Gateway', 'smartpay') . '</h3>';
                echo '<p>' . __('') . '</p>';
                echo '<table class="form-table">';
                // Generate the HTML For the settings form.
                $this->generate_settings_html();
                echo '</table>';
            }

            /**
             *  There are no payment fields for smartpay, but we want to show the description if set.
             **/
            function payment_fields()
            {
                if ($this->description)
                    echo wpautop(wptexturize($this->description));
            }

            /**
             * Receipt Page
             **/
            function receipt_page($order)
            {
                echo $this->generate_smartpay_form($order);
            }

            /**
             * Generate smartpay button link
             **/
            public function generate_smartpay_form($order_id)
            {
                global $woocommerce;
                $thisDir = dirname(__FILE__);
                $thisFile = $thisDir . '/log.txt';

                $order = new WC_Order($order_id);

                $params                     = array();
                $params['merchantUsername'] = $this->smartpay_merchant_key;
                $params['merchantPassword'] = $this->smartpay_merchant_secret;
                $params['paymentMethod']    = "SC_PAY_USING_CBK";

                $params['redirectTo']             = base64_encode($this->get_return_url($order));
                $params['s2s']                    = base64_encode(get_site_url());
                $params['jsonCart']['cart_id']    = $order_id;
                $params['jsonCart']['total']      = $order->get_total();
                $params['jsonCart']['currency']   = "EUR"; //$order->currency;
                $params['jsonCart']['shipping']   = $order->get_shipping_total();
                $params['jsonCart']['additional'] = $order->get_total_tax();
                $params['jsonCart']['discount']   = $order->get_total_discount();

                $params['jsonCart']['values'] = array(
                    array(
                        'title' => "Payment",
                        'description' => " ",
                        'quantity' => 1,
                        'image' => "",
                        'amount' => $order->get_total()
                    )
                );

                $params['jsonCart'] = base64_encode(json_encode($params['jsonCart']));

                $authUrl = $this->initializePaymentUrl . '?' . http_build_query($params);

                //$result = file_get_contents($authUrl);
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $authUrl);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
                curl_setopt($ch, CURLOPT_TIMEOUT, 30);



                    if($this->smartpay_proxy_active){
                curl_setopt($ch, CURLOPT_PROXY, $this->smartpay_proxy_url);     // PROXY details with port
                curl_setopt($ch, CURLOPT_PROXYTYPE, SC_PROXY_TYPE);
                curl_setopt($ch, CURLOPT_PROXYUSERPWD, $this->smartpay_proxy_login.":".$this->smartpay_proxy_password);   // Use if proxy have username and password
                    }



                $result = curl_exec($ch);
                if (curl_errno($ch)) {
                    file_put_contents($thisFile, $authUrl.' & ' . json_encode(curl_error($ch)) . "\n", FILE_APPEND);
                }
                curl_close($ch);

                $result = json_decode($result, 1);

                //DEBUG file_put_contents($thisFile, '& '.json_encode($result)."\n", FILE_APPEND);
                //DEBUG file_put_contents($thisFile, '&& '.$authUrl."\n", FILE_APPEND);
                //file_put_contents($thisFile, '&&& '.$this->get_return_url($order)."\n", FILE_APPEND);

                if (isset($result['response']) && isset($result['response']['transaction_status']) && $result['response']['transaction_status'] == 0) {
                    ob_clean();
                    if (isset($result['response']['transaction_message'])) {
                        wp_die("SmartPay Error: " . $result['response']['transaction_message'], 'error');
                    } else {
                        wp_die("SmartPay - Server connection error", 'error');
                    }
                    return;
                }
                print("<script type = 'text/javascript'> location.href = '" . $result['response']['transaction_action_url'] . "';</script>");
                return;
            }

            /**
             * Process the payment and return the result
             **/
            function order_completed_callback($order_status, $order_id)
            {
                $order = wc_get_order($order_id);

                $thisDir = dirname(__FILE__);
                $thisFile = $thisDir . '/log.txt';

                //DEBUG file_put_contents($thisFile, '%% order_completed_callback: ' . $order_status . "\n", FILE_APPEND);

                if ($order && $order->get_status() !== 'completed') {
                    $order->update_status('completed');
                }
            }

            function process_payment($order_id)
            {
                global $woocommerce;
                $thisDir = dirname(__FILE__);
                $thisFile = $thisDir . '/log.txt';
                $order = new WC_Order($order_id);

                $payment_page = get_permalink(woocommerce_get_page_id('pay'));
                //DEBUG file_put_contents($thisFile, 'Redirect URL: ' . $payment_page . "\n", FILE_APPEND);

                if (false && get_option('woocommerce_force_ssl_checkout') == 'yes') {
                    $payment_page = str_replace('http:', 'https:', $payment_page);
                }
                $order->update_status('pending');

                return array(
                    'result' => 'success',
                    'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, $payment_page))
                );
            }


            /**
             * Check for valid smartpay server callback
             **/
            function check_smartpay_response()
            {
                global $woocommerce;

                $thisDir = dirname(__FILE__);
                $thisFile = $thisDir . '/log.txt';

                $transaction_token  = (isset($_REQUEST['sc_transaction_token']) && !empty($_REQUEST['sc_transaction_token'])) ? $_REQUEST['sc_transaction_token'] : null;
                $transaction_status = (isset($_REQUEST['sc_transaction_status']) && !empty($_REQUEST['sc_transaction_status'])) ? $_REQUEST['sc_transaction_status'] : null;
                $sc_cart_id         = (isset($_REQUEST['sc_cart_id']) && !empty($_REQUEST['sc_cart_id'])) ? $_REQUEST['sc_cart_id'] : null;

                //DEBUG file_put_contents($thisFile, '# '.json_encode($_REQUEST, true)."\n", FILE_APPEND);

                if (isset($transaction_token)) {
                    //launch service
                    $confirmActionUrl           = $this->getTransactionStatusUrl;
                    $params                     = array();
                    $params['merchantUsername'] = $this->smartpay_merchant_key;
                    $params['merchantPassword'] = $this->smartpay_merchant_secret;
                    $params['transactionToken'] = $transaction_token;

                    $confirmActionUrl .= "?" . http_build_query($params);

                    //$apiResult = file_get_contents($confirmActionUrl);
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $confirmActionUrl);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
                    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
                    
                         if($this->smartpay_proxy_active){
                curl_setopt($ch, CURLOPT_PROXY, $this->smartpay_proxy_url);     // PROXY details with port
                curl_setopt($ch, CURLOPT_PROXYTYPE, SC_PROXY_TYPE);
                curl_setopt($ch, CURLOPT_PROXYUSERPWD, $this->smartpay_proxy_login.":".$this->smartpay_proxy_password);   // Use if proxy have username and password
                    }




                    $apiResult = curl_exec($ch);
                    if (curl_errno($ch)) {
                        file_put_contents($thisFile, '& ' . json_encode(curl_error($ch)) . "\n", FILE_APPEND);
                    }
                    curl_close($ch);

                    //DEBUG file_put_contents($thisFile, '## '.json_encode($apiResult, true)."\n", FILE_APPEND);

                    if ($apiResult) {
                        $response = json_decode($apiResult, 1);
                        //DEBUG file_put_contents($thisFile, '### '.json_encode($response, true)."\n", FILE_APPEND);

                        if ($response && isset($response['response']['cart_id']) && !empty($response['response']['cart_id'])) {

                            $order_id = $response['response']['cart_id'];
                            //DEBUG file_put_contents($thisFile, '#### '.json_encode($order_id, true)."\n", FILE_APPEND);

                            if (($order_id) == null) {
                                $this->msg['class']   = 'woocommerce_error';
                                $this->msg['message'] = "Thank you for shopping with us. However, the transaction has been declined.";
                                add_action('the_content', array(&$this, 'showMessage'));
                                return;
                            }

                            $order = new WC_Order($order_id);
                            //DEBUG file_put_contents($thisFile, '$ '.json_encode($order, true)."\n", FILE_APPEND);

                            if ($order && $order->get_status() == 'pending') {
                                if ($response['response']['payed'] == 1) {
                                    $isAuthorized         = true;
                                    $this->msg['message'] = "Thank you for shopping with us. Your account has been charged and your transaction is successful. We will be shipping your order to you soon.";
                                    $this->msg['class']   = 'woocommerce_message';

                                    if ($order->get_status() != 'processing') {
                                        //DEBUG file_put_contents($thisFile, "PROCESSING\n", FILE_APPEND);
                                        //$order->payment_complete();

                                        $new_status = 'processing'; // << https://woo.com/document/managing-orders/
                                        $order->add_order_note($new_status);
                                        $order->update_status($new_status);

                                        $order->add_order_note('Payment Completed. Transaction Code: ' . $transaction_token);
                                        $order->add_order_note($this->msg['message']);
                                        $woocommerce->cart->empty_cart();
                                    }
                                } else {

                                    $order->update_status('failed');
                                    $order->add_order_note('Failed');
                                    $order->add_order_note($this->msg['message']);
                                }
                            }
                        }
                    }
                }
            }

            function showMessage($content)
            {
                return '<div class="box ' . $this->msg['class'] . '-box">' . $this->msg['message'] . '</div>' . $content;
            }
            // get all pages

        }
        /**
         * Add the Gateway to WooCommerce
         **/
        function woocommerce_add_smartpay_gateway($methods)
        {
            $methods[] = 'WC_SmartPay';
            return $methods;
        }
        add_filter('woocommerce_payment_gateways', 'woocommerce_add_smartpay_gateway');
    }
}